#!/usr/bin/python

import unittest
import traceback
from proinfo_inventory import get_connection, clean_inventory, insert_data

def tearDownTest(connection, cur, count):
    clean_inventory(cur, count)
    connection.commit()
    connection.close()


def setUpTest(database):
    connection = get_connection(database, 'postgres')
    cur = connection.cursor()
    inserted = insert_data(cur)
    cur.execute('SELECT load_dw();')
    connection.commit()
    return (connection, cur, inserted)

def test_load_count(cur, inserted):
    print 'All valid machines are loaded'
    cur.execute("SELECT COUNT(*) FROM (SELECT macaddr, sch_id FROM dim_inventory GROUP BY macaddr, sch_id) AS t1;")
    tup = cur.fetchone()
    assert tup[0] == inserted

def test_one_per_day(cur):
    print 'There is only one inventory entry per day'
    cur.execute("SELECT count(*) FROM dim_inventory GROUP BY load_date, sch_id, macaddr;")    
    for tup in cur.fetchmany():
        assert tup[0] == 1

def test_one_per_day_multiple_load(cur):
    print 'There is only one entry per day even if multiple loads happen'
    inserted = insert_data(cur)
    cur.execute("SELECT load_dw();")
    cur.execute("SELECT count(*) FROM dim_inventory GROUP BY load_date, sch_id, macaddr;")    
    for tup in cur.fetchall():
        assert tup[0] == 1

def test_inventory_data(cur):
    print 'All inventory data is correctly inserted'
    cur.execute('SELECT COUNT(*) FROM dim_inventory;')
    total = cur.fetchone()[0]
    cur.execute('''SELECT COUNT(*) FROM (SELECT * FROM dim_inventory
WHERE os_type = %s AND os_distro = %s AND os_kernel = %s 
AND processor = %s AND memory = %s AND disk1_model = %s AND disk1_size = %s
AND disk1_used = %s) AS t1''', 
                ('Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic',
                 'Intel(R) Celeron(R) CPU E1200  @', 160,
                 '1016076 ST3160318AS_6VY4BK6R', 140, 5))
    assert total == cur.fetchone()[0]

def test_inventory_multi_hd(cur, count):
    print 'Inventory is correctly inserted with many hds'
    from proinfo_inventory import (HD_MODEL, HD_SIZE, HD_USED, HD_EXTRA,
                                   MULTI_HD_TOTAL)
    cur.execute('''SELECT COUNT(*) FROM (SELECT macaddr, sch_id FROM dim_inventory
WHERE os_type = %s AND os_distro = %s AND os_kernel = %s 
AND processor = %s AND memory = %s AND disk1_model = %s AND disk1_size = %s
AND disk1_used = %s AND disk2_model = %s AND disk2_size = %s
AND disk2_used = %s GROUP BY macaddr, sch_id) AS t1''',
                ('Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic',
                 'Intel(R) Celeron(R) CPU E1200  @', 160,
                 '1016076 ST3160318AS_6VY4BK6R', 140, 5, HD_MODEL, HD_SIZE, HD_USED))
    matching = cur.fetchone()
    assert MULTI_HD_TOTAL == matching[0]

def test_clean_proinfo_inventory(cur):
    print 'proinfo_inventory is cleared after load'
    cur.execute('SELECT COUNT(*) FROM proinfo_inventory;')
    count = cur.fetchone()[0]
    assert 0 == count

def test_move_incomplete_proinfo_inventory(cur):
    print 'Incomplete inventory entries are moved to rejected_inventory'
    inv = []
    inv.append("INSERT INTO proinfo_inventory(project, inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES (0, 'PR97201405', '0:0:0:0', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(project, inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES (0, 'invalid', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(project, inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES (4, 'PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', null, 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', null, '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', null, 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', null, 160, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', null, '1016076 ST3160318AS_6VY4BK6R', 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, null, 150, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', null, 10);")
    inv.append("INSERT INTO proinfo_inventory(inep, macaddr, os_type, os_distro, os_kernel, processor, memory, disk1_model, disk1_size, disk1_used) VALUES ('PR97201405', '00:00:00:00:00:01', 'Linux', 'Ubuntu 10.04.4 LTS', '2.6.35-25-generic', 'Intel(R) Celeron(R) CPU E1200  @', 160, '1016076 ST3160318AS_6VY4BK6R', 150, null);")

    for i in inv:
        cur.execute(i)

    cur.execute("SELECT COUNT(*) FROM dim_inventory;")
    oldCount = cur.fetchone()[0]
    cur.execute('SELECT load_dw();')
    cur.execute("SELECT COUNT(*) FROM dim_inventory;")
    newCount = cur.fetchone()[0]
    assert newCount == oldCount
    cur.execute("SELECT COUNT(*) FROM rejected_inventory;")
    assert len(inv) == cur.fetchone()[0]

def main():
    from sys import argv, exit
    if len(argv) < 2:
        exit('Usage: {0} <database>'.format(argv[0]))
    conn, cur, count = setUpTest(argv[1])

    try:
        test_clean_proinfo_inventory(cur)
        test_load_count(cur, count)
        test_one_per_day(cur)
        test_inventory_data(cur)
        test_inventory_multi_hd(cur, count)
        test_move_incomplete_proinfo_inventory(cur)
#        test_one_per_day_multiple_load(cur) Probably never going to happen
        print 'Ok'

    except AssertionError:
        print 'There are failed tests ' + traceback.format_exc()
    finally:
        cur.execute('rollback;')
        tearDownTest(conn, cur, count)

if __name__ == '__main__':
    main()
