/*
 * Copyright (C) 2004-2011 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */

package br.ufpr.c3sl.datasid;

import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.math.BigInteger;

import java.sql.Timestamp;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Types;

import javax.naming.InitialContext;
import javax.sql.DataSource;
import javax.xml.bind.*;
import javax.xml.bind.util.ValidationEventCollector;
import javax.xml.bind.helpers.DefaultValidationEventHandler;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

public class DataSID {

    private static final int LINUX = 0;
    private static final int WINDOWS = 1;

    private static final String SA_INVENTORY = "telecenter_inventory";
    private static final String SA_NET_USAGE = "net_usage";
    private static final String SA_USER_HISTORY = "telecenter_user_history";

    private static final File XML_INVENTORY_SCHEMA = new File("/home/datasid/conf/collected-data.xsd");
	private static final File XML_NET_USAGE_SCHEMA = new File("/home/datasid/conf/net-collected-data.xsd");

    private static final String AGENT_VERSION = "1.0.0";
    private static final String AGENT_UPDATE_LINK = "http://bisimmcdev.c3sl.ufpr.br/download/datasid-1.0.0-update.run";

    private static final String WINDOWS_AGENT_VERSION = "1.0.0";
    private static final String WINDOWS_UPDATE_LINK = "http://bisimmcdev.c3sl.ufpr.br/download/datasid-1.0.0-update.exe";

    // enum does not work as expected inside an axis web service
    // using simple constants instead
    private static final int ERROR = 0;
    private static final int WARNING = 1;
    private static final int INFO = 2;
    private static final int DEBUG = 3;

    /**
     * Returns the name of the log level
     *
     * @author                  Eduardo Luis Buratti
     * @param       level       Log level
     */
    private static String logLevelName(int level) {
        if (level == ERROR) return "ERROR";
        else if (level == WARNING) return "WARNING";
        else if (level == INFO) return "INFO";
        else if (level == DEBUG) return "DEBUG";
        else return "NULL";
    }

    /**
     * Write messages to the log
     *
     * @author                  Eduardo Luis Buratti
     * @param       level       Log level
     * @param       msg         Message
     */
    private static void log(int level, String msg) {
        System.out.println(getTimestamp() + " [DataSeedWS][" + logLevelName(level) + "]: " + msg);
    }

    /**
     * Returns a string representing the current date
     *
     * @author                  Eduardo Luis Buratti
     * @return                  String
     */
    private static String getDate() {
        Calendar date = Calendar.getInstance(); // gets current date

        int year = date.get(Calendar.YEAR);
        int month = date.get(Calendar.MONTH);
        int day = date.get(Calendar.DATE);

        /* NOTE: Calendar.MONTH field value is 0-based. e.g: 0 is January */
        month++;

        return year + "-" + month + "-" + day;
    }

    /**
     * Returns current timestamp
     *
     * @author                  Eduardo Luis Buratti
     * @return                  String
     */
    private static String getTimestamp() {
        // gets the current time
        java.util.Date date = new java.util.Date();
        long milisecs = date.getTime();

        // generates timestamp string
        Timestamp timestamp = new Timestamp(milisecs);
        return timestamp.toString();
    }

    /**
     * Return a string with current agent version.
     *
     * @author                  Eduardo Luis Buratti
     * @param       OS          Integer of OS Type
     * @return                  String
     */
    public static String getAgentVersion(int OS) {
        switch (OS) {
            case LINUX:
                return AGENT_VERSION;
            case WINDOWS:
                return WINDOWS_AGENT_VERSION;
            default:
                return "ERROR: invalid OS";
        }
    }

    /**
     * Return a string that contains a link to download the newest version of
     * agent.
     *
     * @author                  Eduardo Luis Buratti
     * @param       OS          Integer of OS Type
     * @return                  String
     */
    public static String getUpdateLink(int OS)
    {
        switch (OS) {
            case LINUX:
                return AGENT_UPDATE_LINK;
            case WINDOWS:
                return WINDOWS_UPDATE_LINK;
            default:
                return "ERROR: invalid OS";
        }
    }

    /**
     * Receive an XML string which has the inventory data to be parsed and
     * inserted into database. Return "Success" string if insertion operation
     * is successful. Any other errors will throw exceptions.
     *
     * @author                 Eduardo Luis Buratti
     * @param     xmlData      XML string of inventory
     * @return                 String
     */
    public static String setInventory(String xmlData) {
        try {
            InitialContext cxt = new InitialContext();
            DataSource ds = (DataSource) cxt.lookup("java:/comp/env/jdbc/datasid");
            if (ds == null)
                throw new Exception("Data source not found!");

            Connection con = ds.getConnection();
            if (con == null)
                throw new Exception("Failed to get a database connection!");

            SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
            Schema schema = factory.newSchema(XML_INVENTORY_SCHEMA);

            JAXBContext context = JAXBContext.newInstance("br.ufpr.c3sl.datasid");
            Unmarshaller unmarshaller = context.createUnmarshaller();
            unmarshaller.setSchema(schema);

            // Strip spaces in the beginning of the xml
            xmlData = xmlData.replaceAll("^\\s+", "");

            // transform the xml string into a InputStream
            InputStream is = new ByteArrayInputStream(xmlData.getBytes());

            // Decode the XML into a Java Object
            JAXBElement<CollectedData> element = (JAXBElement<CollectedData>) unmarshaller.unmarshal(is);
            CollectedData collected = element.getValue();

            Calendar cal = Calendar.getInstance();
            java.sql.Date contactDate = new java.sql.Date(cal.getTimeInMillis());

            PreparedStatement st = createInventoryStatement(con, collected, contactDate);
            st.executeUpdate();

			/*List<User> users = collected.getUserHistory().getUser();
			String superid = collected.getTelecentroInfo().getIdPoint();*/
			List<Interface> interfaces = collected.getInterfaces().getInterface();
			org.postgresql.util.PGobject macaddr = new org.postgresql.util.PGobject();
        	macaddr.setType("macaddr");
        	macaddr.setValue(interfaces.get(0).getMacAddress());

			/*for (User user: users) {
				st = createUserHistoryStatement(con, contactDate, superid, macaddr, user.getName(), user.getLogin(), user.getLogout());
				st.executeUpdate();
			}*/

            con.close();

            log(INFO, "setInventory(id_point=" + collected.getTelecentroInfo().getIdPoint() + ", macaddr=" + interfaces.get(0).getMacAddress() + ")");
            return "Success";
        } catch (Exception e) {
            log(ERROR, e.getMessage() + " " + xmlData);
            e.printStackTrace();
            return "ERROR: " + e.getMessage();
        }
    }

	private static PreparedStatement createInventoryStatement(Connection con, CollectedData collectedData, java.sql.Date contactDate) throws SQLException {
        final String query = "INSERT INTO " + SA_INVENTORY + " " +
            "(contact_date, machine_type, id_point, macaddr, agent_version" +
            " os_type, os_distro, os_kernel, " +
            " processor, memory, " +
            " disk1_model, disk1_size, disk1_used, " +
            " disk2_model, disk2_size, disk2_used, " +
            " extra_hds, " +
            " mirror_timestamp, " +
            ") VALUES " +
            "(?, ?, ?, ?, ?, " +
            " ?, ?, ?, " +
            " ?, ?, " +
            " ?, ?, ?, " +
            " ?, ?, ?, " +
            " ?, " +
            " ?);";

        PreparedStatement st = con.prepareStatement(query);

        List<Interface> interfaces = collectedData.getInterfaces().getInterface();

        Inventory inventory = collectedData.getInventory();
        List<Disk> disks = inventory.getDisks().getDisk();
        TeleCentroInfo teleCentroInfo = collectedData.getTelecentroInfo();

        // contact_date
       	st.setDate(1, contactDate);

        // machine_type
        if(collectedData.getMachineType().compareTo("client") == 0)
            st.setInt(2, 0);
        else
            st.setInt(2, 1);

        // id_point
        st.setInt(3, teleCentroInfo.getIdPoint().intValue());

        // macaddr
        org.postgresql.util.PGobject macaddr = new org.postgresql.util.PGobject();
        macaddr.setType("macaddr");
        macaddr.setValue(interfaces.get(0).getMacAddress());
        st.setObject(4, macaddr);

	    // agent_version
        st.setString(5, collectedData.getAgentVersion());

        // os_type
        st.setString(6, inventory.getOs());

        // os_distro
        st.setString(7, inventory.getDistro());

        // os_kernel
        st.setString(8, inventory.getKernel());

        // processor
        st.setString(9, inventory.getProcessor());

        // memory
        st.setInt(10, inventory.getMemory().intValue());

        // disk1_model
        st.setString(11, disks.get(0).getModel());

        // disk1_size
        st.setInt(12, disks.get(0).getSize().intValue());

        // disk1_used
        st.setInt(13, disks.get(0).getUsed().intValue());

        if (disks.size() > 1) {
            // disk2_model
            st.setString(14, disks.get(1).getModel());

            // disk2_size
            st.setInt(15, disks.get(1).getSize().intValue());

            // disk2_used
            st.setInt(16, disks.get(1).getUsed().intValue());
        }
        else {
            // disk2_model
            st.setNull(14, Types.VARCHAR);

            // disk2_size
            st.setNull(15, Types.INTEGER);

            // disk2_used
            st.setNull(16, Types.INTEGER);
        }

        // extra_hds
        st.setInt(17, (disks.size() > 2) ? (disks.size() - 2) : 0);

        // mirrors_timestamp
        st.setString(18, collectedData.getMirrorsTimestamp());

        // user_count
        //st.setInt(???, teleCentroInfo.getUserCount().intValue());

        return st;
    }

    private static PreparedStatement createUserHistoryStatement(Connection con, java.sql.Date contactDate,
            int id_point, Object macaddr, String name, String login, String logout) throws SQLException {
    	final String query = "INSERT INTO " + SA_USER_HISTORY + " " +
            "(contact_date, m_superid, macaddr, name, login, logout) VALUES " +
            "(?, ?, ?, ?, ?, ?);";

        PreparedStatement st = con.prepareStatement(query);

        st.setDate(1, contactDate);
   		
        st.setInt(2, id_point);

   		st.setObject(3, macaddr);

   		st.setString(4, name);

   		st.setString(5, login);

   		st.setString(6, logout);

        return st;
    }

    /**
     * Receive an XML string which has the inventory data to be parsed and
     * inserted into database. Return "Success" string if insertion operation
     * is successful. Any other errors will throw exceptions.
     *
     * @author                 Eduardo Luis Buratti
     * @param     xmlData      XML string of inventory
     * @return                 String
     */
    public static String setNetUsage(String xmlData)
    {
        try {
            InitialContext cxt = new InitialContext();
            DataSource ds = (DataSource) cxt.lookup("java:/comp/env/jdbc/datasid");
            if (ds == null)
                throw new Exception("Data source not found!");

            Connection con = ds.getConnection();
            if (con == null)
                throw new Exception("Failed to get a database connection!");

            SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
            Schema schema = factory.newSchema(XML_NET_USAGE_SCHEMA);

            JAXBContext context = JAXBContext.newInstance("br.ufpr.c3sl.datasid");
            Unmarshaller unmarshaller = context.createUnmarshaller();
            unmarshaller.setSchema(schema);

            // Strip spaces in the beginning of the xml
            xmlData = xmlData.replaceAll("^\\s+", "");

            // transform the xml string into a InputStream
            InputStream is = new ByteArrayInputStream(xmlData.getBytes());

            // Decode the XML into a Java Object
            JAXBElement<NetCollectedData> element = (JAXBElement<NetCollectedData>) unmarshaller.unmarshal(is);
            NetCollectedData netCollectedData = element.getValue();

			List<Interface> interfaces = netCollectedData.getInterfaces().getInterface();

       		// contact_date = current date
        	Calendar cal = Calendar.getInstance();
        	java.sql.Date contactDate = new java.sql.Date(cal.getTimeInMillis());
        	// macaddr
        	org.postgresql.util.PGobject macaddr = new org.postgresql.util.PGobject();
        	macaddr.setType("macaddr");
        	macaddr.setValue(interfaces.get(0).getMacAddress());

        	List<NetUse> netUses = netCollectedData.getBandwidthUsage().getNetuse();

			for(NetUse netUse : netUses) {
		        PreparedStatement st = createNetUsageStatement(con, contactDate, netCollectedData.getIdPoint(), (Object)macaddr, netUse.getDate(), netUse.getTime(), netUse.getRx().getBytes(), netUse.getRx().getPackets(), netUse.getTx().getBytes(), netUse.getTx().getPackets());
				st.executeUpdate();
			}
            con.close();

            log(DEBUG, "setNetUsage(id_point=" + netCollectedData.getIdPoint() + ", macaddr=" + interfaces.get(0).getMacAddress() + ")");
            return "Success";
        } catch (Exception e) {
            log(ERROR, e.getMessage() + " " + xmlData);
            e.printStackTrace();
            return "ERROR: " + e.getMessage();
        }
    }

    private static PreparedStatement createNetUsageStatement(Connection con, java.sql.Date contactDate,
            BigInteger id_point, Object macaddr, String collect_date, String collect_time, BigInteger down_kbits,
            BigInteger down_packages, BigInteger up_kbits, BigInteger up_packages) throws SQLException {
        final String query = "INSERT INTO " + SA_NET_USAGE + " " +
            "(contact_date, id_point, macaddr, collect_date, collect_time," +
            "down_kbits, down_packages, up_kbits, up_packages) VALUES " +
            "(?, ?, ?, ?, ?, ?, ?, ?, ?);";

        PreparedStatement st = con.prepareStatement(query);

 		st.setDate(1, contactDate);
   		
        st.setInt(2, id_point.intValue());

   		st.setObject(3, macaddr);

   		st.setString(4, collect_date);

   		st.setString(5, collect_time);

   		st.setInt(6, down_kbits.intValue());

   		st.setInt(7, down_packages.intValue());

   		st.setInt(8, up_kbits.intValue());

   		st.setInt(9, up_packages.intValue());

	    return st;
	}
}
